@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2014-2019 Cisco and/or its affiliates. All rights reserved.
+** Copyright (C) 2014-2021 Cisco and/or its affiliates. All rights reserved.
 ** Copyright (C) 2010-2013 Sourcefire, Inc.
 ** Author: Michael R. Altizer <mialtize@cisco.com>
 **
@@ -31,6 +31,9 @@ extern "C" {
 #include <stdint.h>
 #include <unistd.h>
 
+// Comprehensive version number covering all elements of this header
+#define DAQ_COMMON_API_VERSION  0x00030002
+
 #ifndef DAQ_SO_PUBLIC
 #  ifdef HAVE_VISIBILITY
 #    define DAQ_SO_PUBLIC  __attribute__ ((visibility("default")))
@@ -105,25 +108,24 @@ typedef struct _daq_msg
 } DAQ_Msg_t;
 
 /* The DAQ packet header structure. */
-#define DAQ_PKT_FLAG_OPAQUE_IS_VALID    0x00001  /* The DAQ module actively set the opaque value in the DAQ packet header. */
-#define DAQ_PKT_FLAG_NOT_FORWARDING     0x00002  /* The DAQ module will not be actively forwarding this packet
+#define DAQ_PKT_FLAG_OPAQUE_IS_VALID    0x0001  /* The DAQ module actively set the opaque value in the DAQ packet header. */
+#define DAQ_PKT_FLAG_NOT_FORWARDING     0x0002  /* The DAQ module will not be actively forwarding this packet
                                                     regardless of the verdict (e.g, Passive or Inline Tap interfaces). */
-#define DAQ_PKT_FLAG_PRE_ROUTING        0x00004  /* The packet is being routed via us but packet modifications
+#define DAQ_PKT_FLAG_PRE_ROUTING        0x0004  /* The packet is being routed via us but packet modifications
                                                     (MAC and TTL) have not yet been made. */
-#define DAQ_PKT_FLAG_IGNORE_VLAN        0x00008  /* Ignore vlan tags in the packet */
-#define DAQ_PKT_FLAG_FLOWID_IS_VALID    0x00010  /* The DAQ module actively set the flow ID value in the DAQ packet header. */
-#define DAQ_PKT_FLAG_LOCALLY_DESTINED   0x00020  /* The packet is destined for local delivery */
-#define DAQ_PKT_FLAG_LOCALLY_ORIGINATED 0x00040  /* The packet was originated locally */
-#define DAQ_PKT_FLAG_SCRUBBED_TCP_OPTS  0x00080  /* Scrubbed tcp options may be available */
-#define DAQ_PKT_FLAG_HA_STATE_AVAIL     0x00100  /* HA State is availble for the flow this packet is associated with. */
-#define DAQ_PKT_FLAG_ERROR_PACKET       0x00200  /* Lower level reports that the packet has errors. */
-#define DAQ_PKT_FLAG_RETRY_PACKET       0x00400  /* Packet is from the retry queue. */
-#define DAQ_PKT_FLAG_TRACE_ENABLED      0x00800  /* The packet has been flagged for tracing by the lower layer. */
-#define DAQ_PKT_FLAG_SIMULATED          0x01000  /* Packet is simulated/virtual */
-#define DAQ_PKT_FLAG_NEW_FLOW           0x02000  /* The packet was the first of a new flow. */
-#define DAQ_PKT_FLAG_REV_FLOW           0x04000  /* The packet is going the reverse direction of the flow initiator.*/
-#define DAQ_PKT_FLAG_DEBUG_ENABLED      0x08000  /* The packet has been flagged for debugging by the lower layer. */
-#define DAQ_PKT_FLAG_SIGNIFICANT_GROUPS 0x10000  /* Interface groups should be used for flow classification. */
+#define DAQ_PKT_FLAG_IGNORE_VLAN        0x0008  /* Ignore vlan tags in the packet */
+#define DAQ_PKT_FLAG_FLOWID_IS_VALID    0x0010  /* The DAQ module actively set the flow ID value in the DAQ packet header. */
+#define DAQ_PKT_FLAG_LOCALLY_DESTINED   0x0020  /* The packet is destined for local delivery */
+#define DAQ_PKT_FLAG_LOCALLY_ORIGINATED 0x0040  /* The packet was originated locally */
+#define DAQ_PKT_FLAG_SCRUBBED_TCP_OPTS  0x0080  /* Scrubbed tcp options may be available */
+#define DAQ_PKT_FLAG_HA_STATE_AVAIL     0x0100  /* HA State is availble for the flow this packet is associated with. */
+#define DAQ_PKT_FLAG_ERROR_PACKET       0x0200  /* Lower level reports that the packet has errors. */
+#define DAQ_PKT_FLAG_TRACE_ENABLED      0x0400  /* Tracing due to packet trace or capture with trace */
+#define DAQ_PKT_FLAG_SIMULATED          0x0800  /* Packet is simulated/virtual */
+#define DAQ_PKT_FLAG_NEW_FLOW           0x1000  /* The packet was the first of a new flow. */
+#define DAQ_PKT_FLAG_REV_FLOW           0x2000  /* The packet is going the reverse direction of the flow initiator.*/
+#define DAQ_PKT_FLAG_DEBUG_ENABLED      0x4000  /* The packet has been flagged for debugging by the lower layer. */
+#define DAQ_PKT_FLAG_SIGNIFICANT_GROUPS 0x8000  /* Interface groups should be used for flow classification. */
 
 #define DAQ_PKTHDR_UNKNOWN  -1  /* Ingress or Egress not known */
 #define DAQ_PKTHDR_FLOOD    -2  /* Egress is flooding */
@@ -136,7 +138,7 @@ typedef struct _daq_pkt_hdr
     int16_t ingress_group;      /* Index of the inbound group. */
     int16_t egress_group;       /* Index of the outbound group. */
     uint32_t opaque;            /* Opaque context value from the DAQ module or underlying hardware.
-                                    Directly related to the opaque value in FlowStats. */
+                                    Directly related to the opaque value in DAQ_FlowStats_t. */
     uint32_t flow_id;           /* Flow ID value provided from the DAQ module or underlying hardware. */
     uint32_t flags;             /* Flags for the packet (DAQ_PKT_FLAG_*) */
     uint16_t address_space_id;  /* Unique ID of the address space */
@@ -163,10 +165,12 @@ typedef struct _daq_napt_info
     to the first protocol of each layer encountered (no information is conveyed about encapsulated
     duplicate protocols like IP-in-IP).  The offsets for layers not found are set to
     DAQ_PKT_DECODE_OFFSET_INVALID. */
-typedef union {
+typedef union _daq_pkt_decode_flags
+{
     uint32_t all;
 
-    struct {
+    struct
+    {
         uint32_t l2:1;              /* Parsed known L2 protocol */
         uint32_t l2_checksum:1;     /* L2 checksum was calculated and validated. */
         uint32_t l3:1;              /* Parsed known L3 protocol */
@@ -230,11 +234,13 @@ typedef struct _daq_flow_desc
     /* L2 Information */
     uint16_t vlan_tag;
     /* L3 Information */
-    union {
+    union
+    {
         struct in_addr in_addr;
         struct in6_addr in6_addr;
     } src_addr;
-    union {
+    union
+    {
         struct in_addr in_addr;
         struct in6_addr in6_addr;
     } dst_addr;
@@ -261,52 +267,53 @@ typedef struct _daq_ha_state_data
 } DAQ_HA_State_Data_t;
 
 #define DAQ_FS_FLAG_SIGNIFICANT_GROUPS 0x1
+#define DAQ_FS_FLAG_LOGGING_OPTIONAL   0x2
 
 /* Flow statistics structure used for DAQ_MSG_TYPE_SOF and DAQ_MSG_TYPE_EOF. */
-typedef struct _flow_stats
+typedef struct _daq_flow_stats
 {
-    int16_t ingressGroup;
-    int16_t egressGroup;
-    int32_t ingressIntf;
-    int32_t egressIntf;
+    int16_t ingress_group;
+    int16_t egress_group;
+    int32_t ingress_intf;
+    int32_t egress_intf;
     /* The IP addresses should be IPv6 or IPv6 representation of IPv4 (::FFFF:<ipv4>) */
-    uint8_t initiatorIp[16];
-    uint8_t responderIp[16];
-    uint16_t initiatorPort;
-    uint16_t responderPort;
+    uint8_t initiator_ip[16];
+    uint8_t responder_ip[16];
+    uint16_t initiator_port;
+    uint16_t responder_port;
     uint32_t opaque;
-    uint64_t initiatorPkts;         /* Not populated for SoF stats. */
-    uint64_t responderPkts;         /* Not populated for SoF stats. */
-    uint64_t initiatorBytes;        /* Not populated for SoF stats. */
-    uint64_t responderBytes;        /* Not populated for SoF stats. */
+    uint64_t initiator_pkts;            /* Not populated for SoF stats. */
+    uint64_t responder_pkts;            /* Not populated for SoF stats. */
+    uint64_t initiator_bytes;           /* Not populated for SoF stats. */
+    uint64_t responder_bytes;           /* Not populated for SoF stats. */
     /* QoS related variables */
-    uint64_t initiatorPktsDropped;  /* Not populated for SoF stats. */
-    uint64_t responderPktsDropped;  /* Not populated for SoF stats. */
-    uint64_t initiatorBytesDropped; /* Not populated for SoF stats. */
-    uint64_t responderBytesDropped; /* Not populated for SoF stats. */
-    uint8_t isQoSAppliedOnSrcIntf;  /* Not populated for SoF stats. */
+    uint64_t initiator_pkts_dropped;    /* Not populated for SoF stats. */
+    uint64_t responder_pkts_dropped;    /* Not populated for SoF stats. */
+    uint64_t initiator_bytes_dropped;   /* Not populated for SoF stats. */
+    uint64_t responder_bytes_dropped;   /* Not populated for SoF stats. */
+    uint8_t is_qos_applied_on_src_intf; /* Not populated for SoF stats. */
     struct timeval sof_timestamp;
-    struct timeval eof_timestamp;   /* Not populated for SoF stats. */
+    struct timeval eof_timestamp;       /* Not populated for SoF stats. */
     uint16_t vlan_tag;
     uint16_t address_space_id;
     uint8_t protocol;
     uint8_t flags;
-} Flow_Stats_t;
+} DAQ_FlowStats_t;
 
 /* Packet verdicts passed to daq_msg_finalize(). */
-typedef enum {
+typedef enum
+{
     DAQ_VERDICT_PASS,       /* Pass the packet. */
     DAQ_VERDICT_BLOCK,      /* Block the packet. */
     DAQ_VERDICT_REPLACE,    /* Pass a packet that has been modified in-place. (No resizing allowed!) */
     DAQ_VERDICT_WHITELIST,  /* Pass the packet and fastpath all future packets in the same flow systemwide. */
     DAQ_VERDICT_BLACKLIST,  /* Block the packet and block all future packets in the same flow systemwide. */
     DAQ_VERDICT_IGNORE,     /* Pass the packet and fastpath all future packets in the same flow for this application. */
-    DAQ_VERDICT_RETRY,      /* Hold the packet briefly and resend it to Snort while Snort waits for external response.
-                               Drop any new packets received on that flow while holding before sending them to Snort. */
     MAX_DAQ_VERDICT
 } DAQ_Verdict;
 
-typedef enum {
+typedef enum
+{
     DAQ_MODE_NONE,
     DAQ_MODE_PASSIVE,
     DAQ_MODE_INLINE,
@@ -323,7 +330,8 @@ typedef struct _daq_variable_desc
     uint32_t flags;
 } DAQ_VariableDesc_t;
 
-typedef enum {
+typedef enum
+{
     DAQ_STATE_UNINITIALIZED,
     DAQ_STATE_INITIALIZED,
     DAQ_STATE_STARTED,
@@ -370,16 +378,15 @@ typedef struct _daq_msg_pool_info
 #define DAQ_CAPA_BPF            0x00000080   /* can call set_filter() to establish a BPF */
 #define DAQ_CAPA_DEVICE_INDEX   0x00000100   /* can consistently fill the device_index field in DAQ_PktHdr */
 #define DAQ_CAPA_INJECT_RAW     0x00000200   /* injection of raw packets (no layer-2 headers) */
-#define DAQ_CAPA_RETRY          0x00000400   /* resend packet to Snort after brief delay. */
-#define DAQ_CAPA_DECODE_GTP     0x00000800   /* decodes and tracks flows within GTP. */
-#define DAQ_CAPA_DECODE_TEREDO  0x00001000   /* decodes and tracks flows within Teredo. */
-#define DAQ_CAPA_DECODE_GRE     0x00002000   /* decodes and tracks flows within GRE. */
-#define DAQ_CAPA_DECODE_4IN4    0x00004000   /* decodes and tracks flows of IPv4 within IPv4. */
-#define DAQ_CAPA_DECODE_6IN4    0x00008000   /* decodes and tracks flows of IPv6 within IPv4. */
-#define DAQ_CAPA_DECODE_4IN6    0x00010000   /* decodes and tracks flows of IPv4 within IPv6. */
-#define DAQ_CAPA_DECODE_6IN6    0x00020000   /* decodes and tracks flows of IPv6 within IPv6. */
-#define DAQ_CAPA_DECODE_MPLS    0x00040000   /* decodes and tracks flows within MPLS. */
-#define DAQ_CAPA_DECODE_VXLAN   0x00080000   /* decodes and tracks flows within VXLAN. */
+#define DAQ_CAPA_DECODE_GTP     0x00000400   /* decodes and tracks flows within GTP. */
+#define DAQ_CAPA_DECODE_TEREDO  0x00000800   /* decodes and tracks flows within Teredo. */
+#define DAQ_CAPA_DECODE_GRE     0x00001000   /* decodes and tracks flows within GRE. */
+#define DAQ_CAPA_DECODE_4IN4    0x00002000   /* decodes and tracks flows of IPv4 within IPv4. */
+#define DAQ_CAPA_DECODE_6IN4    0x00004000   /* decodes and tracks flows of IPv6 within IPv4. */
+#define DAQ_CAPA_DECODE_4IN6    0x00008000   /* decodes and tracks flows of IPv4 within IPv6. */
+#define DAQ_CAPA_DECODE_6IN6    0x00010000   /* decodes and tracks flows of IPv6 within IPv6. */
+#define DAQ_CAPA_DECODE_MPLS    0x00020000   /* decodes and tracks flows within MPLS. */
+#define DAQ_CAPA_DECODE_VXLAN   0x00040000   /* decodes and tracks flows within VXLAN. */
 
 /*
  * DAQ I/O Controls (DIOCTLs)
@@ -503,7 +510,7 @@ typedef enum
     DAQ_TCP_OPTS_TS_CHANGED = 0x08,
 } DAQ_TCP_Opts_flags_t;
 
-typedef struct
+typedef struct _daq_tcp_opts
 {
     uint8_t flags;                  // DAQ_TCP_OPTS_*
     uint8_t window_scale;
@@ -530,14 +537,17 @@ typedef struct
 #define DAQ_EFLOW_TUNNEL_TYPE_GTP_TUNNEL    1
 #define DAQ_EFLOW_TUNNEL_TYPE_MPLS_TUNNEL   2
 #define DAQ_EFLOW_TUNNEL_TYPE_OTHER_TUNNEL  3
-typedef struct _DAQ_EFlow_Key_t {
+typedef struct _daq_eflow_key
+{
     uint16_t src_af;                /* AF_INET or AF_INET6 */
     uint16_t dst_af;                /* AF_INET or AF_INET6 */
-    union {
+    union
+    {
         struct in_addr src_ip4;
         struct in6_addr src_ip6;
     } sa;
-    union {
+    union
+    {
         struct in_addr dst_ip4;
         struct in6_addr dst_ip6;
     } da;
@@ -553,7 +563,7 @@ typedef struct _DAQ_EFlow_Key_t {
 #define DAQ_EFLOW_FLOAT             0x01 /* the expected flow can float to a different reader */
 #define DAQ_EFLOW_ALLOW_MULTIPLE    0x02 /* allow multiple connections to use the same expected flow entry */
 #define DAQ_EFLOW_PERSIST           0x04 /* expected flow entry persists even if control channel terminates */
-typedef struct _DAQ_EFlow_Setup_t
+typedef struct
 {
     DAQ_Msg_h ctrl_msg;     // [in] Message containing the companion control channel packet
     DAQ_EFlow_Key_t key;    // [in] Flow key describing the expected flow
