@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2014-2019 Cisco and/or its affiliates. All rights reserved.
+** Copyright (C) 2014-2021 Cisco and/or its affiliates. All rights reserved.
 ** Copyright (C) 2010-2013 Sourcefire, Inc.
 ** Author: Michael R. Altizer <mialtize@cisco.com>
 **
@@ -24,6 +24,7 @@
 #endif
 
 #include <arpa/inet.h>
+#include <ctype.h>
 #include <errno.h>
 #include <grp.h>
 #include <inttypes.h>
@@ -96,7 +97,8 @@ typedef struct _DAQTestConfig
     bool list_and_exit;
     bool modify_opaque_value;
     bool performance_mode;
-    bool dump_packets;
+    bool dump_hex;
+    bool dump_ascii;
     bool ignore_checksum_errors;
     bool explicit_thread_count;
 } DAQTestConfig;
@@ -150,6 +152,9 @@ extern const DAQ_ModuleAPI_t pcap_daq_module_data;
 #ifdef BUILD_NETMAP_MODULE
 extern const DAQ_ModuleAPI_t netmap_daq_module_data;
 #endif
+#ifdef BUILD_SAVEFILE_MODULE
+extern const DAQ_ModuleAPI_t savefile_daq_module_data;
+#endif
 #ifdef BUILD_TRACE_MODULE
 extern const DAQ_ModuleAPI_t trace_daq_module_data;
 #endif
@@ -180,6 +185,9 @@ static DAQ_Module_h static_modules[] =
 #ifdef BUILD_NETMAP_MODULE
     &netmap_daq_module_data,
 #endif
+#ifdef BUILD_SAVEFILE_MODULE
+    &savefile_daq_module_data,
+#endif
 #ifdef BUILD_TRACE_MODULE
     &trace_daq_module_data,
 #endif
@@ -229,7 +237,8 @@ static void usage(void)
     printf("  -u <username>     Run as the specified user after initialization (accepts UID)\n");
     printf("  -v                Increase the verbosity level of the DAQ library (may be specified multiple times)\n");
     printf("  -V <verdict>      Specify a default verdict to render on packets (pass (default), block, blacklist, whitelist)\n");
-    printf("  -x                Print a hexdump of each packet received\n");
+    printf("  -x                Dump message data in hex\n");
+    printf("  -X                Dump message data in hex and ASCII\n");
     printf("  -z <num>          Specify the number of packet threads to run (default = 1)\n");
 }
 
@@ -238,19 +247,37 @@ static void print_mac(const uint8_t *addr)
     printf("%.2hhx:%.2hhx:%.2hhx:%.2hhx:%.2hhx:%.2hhx", addr[0], addr[1], addr[2], addr[3], addr[4], addr[5]);
 }
 
-static void print_hex_dump(const uint8_t *data, unsigned len)
+#define HEXDUMP_BYTES_PER_LINE 16
+static void hexdump(const uint8_t *data, unsigned len, bool ascii)
 {
-    unsigned i;
+    char text[HEXDUMP_BYTES_PER_LINE + 1];
+    unsigned i = 0;
 
-    for (i = 0; i < len; i++)
+    printf("\n");
+    while (i < len)
     {
-        if (i % 16 == 0)
-            printf("\n");
-        else if (i % 2 == 0)
+        if (i % 2 == 0)
             printf(" ");
         printf("%02x", data[i]);
+        if (ascii)
+            text[i % HEXDUMP_BYTES_PER_LINE] = isprint(data[i]) ? data[i] : '.';
+        i++;
+        if (i % HEXDUMP_BYTES_PER_LINE == 0 || i == len)
+        {
+            if (ascii)
+            {
+                text[(i - 1) % HEXDUMP_BYTES_PER_LINE] = '\0';
+                while (i % HEXDUMP_BYTES_PER_LINE != 0)
+                {
+                    printf("%*s", (i % 2) ? 3 : 2, "");
+                    i++;
+                }
+                printf("  %s", text);
+            }
+            printf("\n");
+        }
     }
-    printf("\n\n");
+    printf("\n");
 }
 
 static void initialize_static_data(void)
@@ -292,10 +319,7 @@ static int replace_icmp_data(DAQTestPacket *dtp)
         data += offset;
         dlen -= offset;
     }
-/*
-    printf("%d bytes of data:\n", dlen);
-    print_hex_dump(data, dlen);
-*/
+
     if (memcmp(data, normal_ping_data + offset, dlen) == 0)
     {
         printf("Replacing the ping request padding.\n");
@@ -785,8 +809,6 @@ static DAQ_Verdict handle_packet_message(DAQTestThreadContext *ctxt, DAQ_Msg_h m
             printf("HA_STATE_AVAIL ");
         if (hdr->flags & DAQ_PKT_FLAG_ERROR_PACKET)
             printf("ERROR_PACKET ");
-        if (hdr->flags & DAQ_PKT_FLAG_RETRY_PACKET)
-            printf("RETRY_PACKET ");
         if (hdr->flags & DAQ_PKT_FLAG_TRACE_ENABLED)
             printf("TRACE_ENABLED ");
         if (hdr->flags & DAQ_PKT_FLAG_SIMULATED)
@@ -866,8 +888,8 @@ static DAQ_Verdict handle_packet_message(DAQTestThreadContext *ctxt, DAQ_Msg_h m
     if (ptad)
         printf("TCP ACK Data: SN = %u, WS = %hu\n", ptad->tcp_ack_seq_num, ptad->tcp_window_size);
 
-    if (cfg->dump_packets)
-        print_hex_dump(data, daq_msg_get_data_len(msg));
+    if (cfg->dump_hex)
+        hexdump(data, data_len, cfg->dump_ascii);
 
     if (cfg->modify_opaque_value)
     {
@@ -895,7 +917,7 @@ static void handle_flow_stats_message(DAQTestThreadContext *ctxt, DAQ_Msg_h msg)
     if (cfg->performance_mode)
         return;
 
-    const Flow_Stats_t *stats = (const Flow_Stats_t *) daq_msg_get_hdr(msg);
+    const DAQ_FlowStats_t *stats = (const DAQ_FlowStats_t *) daq_msg_get_hdr(msg);
     char addr_str[INET6_ADDRSTRLEN];
     const struct in6_addr* tmpIp;
     struct tm tm;
@@ -903,21 +925,21 @@ static void handle_flow_stats_message(DAQTestThreadContext *ctxt, DAQ_Msg_h msg)
 
     printf("\nReceived %s message.\n", msg->type == DAQ_MSG_TYPE_SOF ? "SoF" : "EoF");
 
-    if (stats->ingressIntf != DAQ_PKTHDR_UNKNOWN || stats->ingressGroup != DAQ_PKTHDR_UNKNOWN)
+    if (stats->ingress_intf != DAQ_PKTHDR_UNKNOWN || stats->ingress_group != DAQ_PKTHDR_UNKNOWN)
     {
         printf("  Ingress:\n");
-        if (stats->ingressIntf != DAQ_PKTHDR_UNKNOWN)
-            printf("    Interface: %d\n", stats->ingressIntf);
-        if (stats->ingressGroup != DAQ_PKTHDR_UNKNOWN)
-            printf("    Group: %hd\n", stats->ingressGroup);
+        if (stats->ingress_intf != DAQ_PKTHDR_UNKNOWN)
+            printf("    Interface: %d\n", stats->ingress_intf);
+        if (stats->ingress_group != DAQ_PKTHDR_UNKNOWN)
+            printf("    Group: %hd\n", stats->ingress_group);
     }
-    if (stats->egressIntf != DAQ_PKTHDR_UNKNOWN || stats->egressGroup != DAQ_PKTHDR_UNKNOWN)
+    if (stats->egress_intf != DAQ_PKTHDR_UNKNOWN || stats->egress_group != DAQ_PKTHDR_UNKNOWN)
     {
         printf("  Egress:\n");
-        if (stats->egressIntf != DAQ_PKTHDR_UNKNOWN)
-            printf("    Interface: %d\n", stats->egressIntf);
-        if (stats->egressGroup != DAQ_PKTHDR_UNKNOWN)
-            printf("    Group: %hd\n", stats->egressGroup);
+        if (stats->egress_intf != DAQ_PKTHDR_UNKNOWN)
+            printf("    Interface: %d\n", stats->egress_intf);
+        if (stats->egress_group != DAQ_PKTHDR_UNKNOWN)
+            printf("    Group: %hd\n", stats->egress_group);
     }
     printf("  Protocol: %hhu\n", stats->protocol);
     if (stats->vlan_tag != 0)
@@ -932,7 +954,7 @@ static void handle_flow_stats_message(DAQTestThreadContext *ctxt, DAQ_Msg_h msg)
         printf("\n");
     }
     printf("  Initiator:\n");
-    tmpIp = (const struct in6_addr*)stats->initiatorIp;
+    tmpIp = (const struct in6_addr*)stats->initiator_ip;
     if (tmpIp->s6_addr32[0] || tmpIp->s6_addr32[1] || tmpIp->s6_addr16[4] || tmpIp->s6_addr16[5] != 0xFFFF)
         inet_ntop(AF_INET6, tmpIp, addr_str, sizeof(addr_str));
     else
@@ -940,12 +962,12 @@ static void handle_flow_stats_message(DAQTestThreadContext *ctxt, DAQ_Msg_h msg)
     printf("    IP: %s", addr_str);
     if (stats->protocol == IPPROTO_UDP || stats->protocol == IPPROTO_TCP
             || stats->protocol == IPPROTO_ICMP || stats->protocol == IPPROTO_ICMPV6)
-        printf(":%d", ntohs(stats->initiatorPort));
+        printf(":%d", ntohs(stats->initiator_port));
     printf("\n");
     if (msg->type == DAQ_MSG_TYPE_EOF)
-        printf("    Sent: %" PRIu64 " bytes (%" PRIu64 " packets)\n", stats->initiatorBytes, stats->initiatorPkts);
+        printf("    Sent: %" PRIu64 " bytes (%" PRIu64 " packets)\n", stats->initiator_bytes, stats->initiator_pkts);
     printf("  Responder:\n");
-    tmpIp = (const struct in6_addr*)stats->responderIp;
+    tmpIp = (const struct in6_addr*)stats->responder_ip;
     if (tmpIp->s6_addr32[0] || tmpIp->s6_addr32[1] || tmpIp->s6_addr16[4] || tmpIp->s6_addr16[5] != 0xFFFF)
         inet_ntop(AF_INET6, tmpIp, addr_str, sizeof(addr_str));
     else
@@ -953,10 +975,10 @@ static void handle_flow_stats_message(DAQTestThreadContext *ctxt, DAQ_Msg_h msg)
     printf("    IP: %s", addr_str);
     if (stats->protocol == IPPROTO_UDP || stats->protocol == IPPROTO_TCP
             || stats->protocol == IPPROTO_ICMP || stats->protocol == IPPROTO_ICMPV6)
-        printf(":%d", ntohs(stats->responderPort));
+        printf(":%d", ntohs(stats->responder_port));
     printf("\n");
     if (msg->type == DAQ_MSG_TYPE_EOF)
-        printf("    Sent: %" PRIu64 " bytes (%" PRIu64 " packets)\n", stats->responderBytes, stats->responderPkts);
+        printf("    Sent: %" PRIu64 " bytes (%" PRIu64 " packets)\n", stats->responder_bytes, stats->responder_pkts);
 
     gmtime_r(&stats->sof_timestamp.tv_sec, &tm);
     strftime(timestr, sizeof(timestr), "%Y-%m-%d %H:%M:%S", &tm);
@@ -1078,7 +1100,7 @@ static int parse_command_line(int argc, char *argv[], DAQTestConfig *cfg)
 {
     DAQTestModuleConfig *dtmc;
     IPv4Addr *ip;
-    const char *options = "A:b:c:C:d:D:f:g:hi:klm:M:OpP:s:t:T:u:vV:xz:";
+    const char *options = "A:b:c:C:d:D:f:g:hi:klm:M:OpP:s:t:T:u:vV:xXz:";
     char *endptr;
     int ch;
 
@@ -1313,7 +1335,12 @@ static int parse_command_line(int argc, char *argv[], DAQTestConfig *cfg)
                 break;
 
             case 'x':
-                cfg->dump_packets = true;
+                cfg->dump_hex = true;
+                break;
+
+            case 'X':
+                cfg->dump_hex = true;
+                cfg->dump_ascii = true;
                 break;
 
             case 'z':
