@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2014-2019 Cisco and/or its affiliates. All rights reserved.
+** Copyright (C) 2014-2021 Cisco and/or its affiliates. All rights reserved.
 ** Copyright (C) 2010-2013 Sourcefire, Inc.
 ** Author: Michael R. Altizer <mialtize@cisco.com>
 **
@@ -505,8 +505,7 @@ static const char *daq_verdict_strings[MAX_DAQ_VERDICT] = {
     "Replace",      // DAQ_VERDICT_REPLACE
     "Whitelist",    // DAQ_VERDICT_WHITELIST
     "Blacklist",    // DAQ_VERDICT_BLACKLIST
-    "Ignore",       // DAQ_VERDICT_IGNORE
-    "Retry"         // DAQ_VERDICT_RETRY
+    "Ignore"        // DAQ_VERDICT_IGNORE
 };
 
 static int trace_daq_msg_finalize(void *handle, const DAQ_Msg_t *msg, DAQ_Verdict verdict)
